@@ -42,6 +42,20 @@
 #define DEFAULT_INPUT_REQUEST_TIMEOUT 120 * 1000
 #define DEFAULT_BROWSER_LAUNCH_TIMEOUT 300 * 1000
 
+static char *default_auto_connect[] = {
+	"wifi",
+	"ethernet",
+	"cellular",
+	NULL
+};
+
+static char *default_blacklist[] = {
+	"vmnet",
+	"vboxnet",
+	"virbr",
+	NULL
+};
+
 static struct {
 	connman_bool_t bg_scan;
 	char **pref_timeservers;
@@ -141,24 +155,13 @@ static void parse_config(GKeyFile *config)
 	char **interfaces;
 	char **str_list;
 	gsize len;
-	static char *default_auto_connect[] = {
-		"wifi",
-		"ethernet",
-		"cellular",
-		NULL
-	};
-	static char *default_blacklist[] = {
-		"vmnet",
-		"vboxnet",
-		"virbr",
-		NULL
-	};
 	int timeout;
 
 	if (config == NULL) {
 		connman_settings.auto_connect =
 			parse_service_types(default_auto_connect, 3);
-		connman_settings.blacklisted_interfaces = default_blacklist;
+		connman_settings.blacklisted_interfaces =
+			g_strdupv(default_blacklist);
 		return;
 	}
 
@@ -234,7 +237,8 @@ static void parse_config(GKeyFile *config)
 	if (error == NULL)
 		connman_settings.blacklisted_interfaces = interfaces;
 	else
-		connman_settings.blacklisted_interfaces = default_blacklist;
+		connman_settings.blacklisted_interfaces =
+			g_strdupv(default_blacklist);
 
 	g_clear_error(&error);
 }
@@ -602,6 +606,7 @@ int main(int argc, char *argv[])
 	g_free(connman_settings.auto_connect);
 	g_free(connman_settings.preferred_techs);
 	g_strfreev(connman_settings.fallback_nameservers);
+	g_strfreev(connman_settings.blacklisted_interfaces);
 
 	g_free(option_debug);
 
