@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2012  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -33,6 +33,12 @@
 #define STATUS_URL_IPV4  "http://ipv4.connman.net/online/status.html"
 #define STATUS_URL_IPV6  "http://ipv6.connman.net/online/status.html"
 
+#define wispr_portal_context_ref(_wp_ctxt) \
+	wispr_portal_context_ref_debug(_wp_ctxt, __FILE__, __LINE__, __func__)
+
+#define wispr_portal_context_unref(_wp_ctxt) \
+	wispr_portal_context_unref_debug(_wp_ctxt, __FILE__, __LINE__, __func__)
+
 struct connman_wispr_message {
 	gboolean has_error;
 	const char *current_element;
@@ -53,7 +59,14 @@ enum connman_wispr_result {
 	CONNMAN_WISPR_RESULT_FAILED  = 3,
 };
 
+struct wispr_route {
+	char *address;
+	int if_index;
+};
+
 struct connman_wispr_portal_context {
+	int refcount;
+
 	struct connman_service *service;
 	enum connman_ipconfig_type type;
 
@@ -75,6 +88,8 @@ struct connman_wispr_portal_context {
 	char *wispr_formdata;
 
 	enum connman_wispr_result wispr_result;
+
+	GSList *route_list;
 };
 
 struct connman_wispr_portal {
@@ -115,14 +130,45 @@ static void connman_wispr_message_init(struct connman_wispr_message *msg)
 	msg->location_name = NULL;
 }
 
+static void free_wispr_routes(struct connman_wispr_portal_context *wp_context)
+{
+	while (wp_context->route_list != NULL) {
+		struct wispr_route *route = wp_context->route_list->data;
+
+		DBG("free route to %s if %d type %d", route->address,
+				route->if_index, wp_context->type);
+
+		switch(wp_context->type) {
+		case CONNMAN_IPCONFIG_TYPE_IPV4:
+			connman_inet_del_host_route(route->if_index,
+					route->address);
+			break;
+		case CONNMAN_IPCONFIG_TYPE_IPV6:
+			connman_inet_del_ipv6_host_route(route->if_index,
+					route->address);
+			break;
+		case CONNMAN_IPCONFIG_TYPE_UNKNOWN:
+			break;
+		}
+
+		g_free(route->address);
+		g_free(route);
+
+		wp_context->route_list =
+			g_slist_delete_link(wp_context->route_list,
+					wp_context->route_list);
+	}
+}
+
 static void free_connman_wispr_portal_context(struct connman_wispr_portal_context *wp_context)
 {
-	DBG("");
+	DBG("context %p", wp_context);
 
 	if (wp_context == NULL)
 		return;
 
-	connman_service_unref(wp_context->service);
+	if (wp_context->service != NULL)
+		connman_service_unref(wp_context->service);
 
 	if (wp_context->token > 0)
 		connman_proxy_lookup_cancel(wp_context->token);
@@ -130,20 +176,81 @@ static void free_connman_wispr_portal_context(struct connman_wispr_portal_contex
 	if (wp_context->request_id > 0)
 		g_web_cancel_request(wp_context->web, wp_context->request_id);
 
-	g_web_unref(wp_context->web);
+	if (wp_context->web != NULL)
+		g_web_unref(wp_context->web);
 
 	g_free(wp_context->redirect_url);
 
-	g_web_parser_unref(wp_context->wispr_parser);
+	if (wp_context->wispr_parser != NULL)
+		g_web_parser_unref(wp_context->wispr_parser);
+
 	connman_wispr_message_init(&wp_context->wispr_msg);
 
 	g_free(wp_context->wispr_username);
 	g_free(wp_context->wispr_password);
 	g_free(wp_context->wispr_formdata);
 
+	free_wispr_routes(wp_context);
+
 	g_free(wp_context);
 }
 
+static struct connman_wispr_portal_context *
+wispr_portal_context_ref_debug(struct connman_wispr_portal_context *wp_context,
+				const char *file, int line, const char *caller)
+{
+	DBG("%p ref %d by %s:%d:%s()", wp_context, wp_context->refcount + 1,
+							file, line, caller);
+
+	__sync_fetch_and_add(&wp_context->refcount, 1);
+
+	return wp_context;
+}
+
+static struct connman_wispr_portal_context *
+wispr_portal_context_unref_debug(struct connman_wispr_portal_context *wp_context,
+				const char *file, int line, const char *caller)
+{
+	DBG("%p ref %d by %s:%d:%s()", wp_context, wp_context->refcount - 1,
+							file, line, caller);
+
+	if (__sync_fetch_and_sub(&wp_context->refcount, 1) != 1)
+		return wp_context;
+
+	free_connman_wispr_portal_context(wp_context);
+
+	return NULL;
+}
+
+static struct connman_wispr_portal_context *create_wispr_portal_context(void)
+{
+	struct connman_wispr_portal_context *wp_context = NULL;
+
+	 wp_context = g_try_new0(struct connman_wispr_portal_context, 1);
+	 if (wp_context == NULL)
+		return NULL;
+
+	 wp_context->refcount = 1;
+
+	 return wp_context;
+}
+
+/**
+ * This function is usued only by free_connman_wispr_portal, context pointer
+ * might still be owned by a third party (gweb, agent dbus call...)
+ * so we don't want service to be referenced in any context when it is not
+ * valid anymore.
+ */
+static void
+reset_service_usage(struct connman_wispr_portal_context *wp_context)
+{
+	if (wp_context->service == NULL)
+		return;
+
+	connman_service_unref(wp_context->service);
+	wp_context->service = NULL;
+}
+
 static void free_connman_wispr_portal(gpointer data)
 {
 	struct connman_wispr_portal *wispr_portal = data;
@@ -153,8 +260,15 @@ static void free_connman_wispr_portal(gpointer data)
 	if (wispr_portal == NULL)
 		return;
 
-	free_connman_wispr_portal_context(wispr_portal->ipv4_context);
-	free_connman_wispr_portal_context(wispr_portal->ipv6_context);
+	if (wispr_portal->ipv4_context != NULL) {
+		reset_service_usage(wispr_portal->ipv4_context);
+		wispr_portal_context_unref(wispr_portal->ipv4_context);
+	}
+
+	if (wispr_portal->ipv6_context != NULL) {
+		reset_service_usage(wispr_portal->ipv6_context);
+		wispr_portal_context_unref(wispr_portal->ipv6_context);
+	}
 
 	g_free(wispr_portal);
 }
@@ -347,7 +461,7 @@ static void xml_wispr_parser_callback(const char *str, gpointer user_data)
 	result = g_markup_parse_context_parse(parser_context,
 					str, strlen(str), NULL);
 	if (result == TRUE)
-		result = g_markup_parse_context_end_parse(parser_context, NULL);
+		g_markup_parse_context_end_parse(parser_context, NULL);
 
 	g_markup_parse_context_free(parser_context);
 }
@@ -389,16 +503,67 @@ static void portal_manage_status(GWebResult *result,
 						wp_context->type);
 }
 
+static gboolean wispr_route_request(const char *address, int ai_family,
+		int if_index, gpointer user_data)
+{
+	int result = -1;
+	struct connman_wispr_portal_context *wp_context = user_data;
+	const char *gateway;
+	struct wispr_route *route;
+
+	gateway = __connman_ipconfig_get_gateway_from_index(if_index,
+		wp_context->type);
+
+	DBG("address %s if %d gw %s", address, if_index, gateway);
+
+	if (gateway == NULL)
+		return FALSE;
+
+	route = g_try_new0(struct wispr_route, 1);
+	if (route == 0) {
+		DBG("could not create struct");
+		return FALSE;
+	}
+
+	switch(wp_context->type) {
+	case CONNMAN_IPCONFIG_TYPE_IPV4:
+		result = connman_inet_add_host_route(if_index, address,
+				gateway);
+		break;
+	case CONNMAN_IPCONFIG_TYPE_IPV6:
+		result = connman_inet_add_ipv6_host_route(if_index, address,
+				gateway);
+		break;
+	case CONNMAN_IPCONFIG_TYPE_UNKNOWN:
+		break;
+	}
+
+	if (result < 0) {
+		g_free(route);
+		return FALSE;
+	}
+
+	route->address = g_strdup(address);
+	route->if_index = if_index;
+	wp_context->route_list = g_slist_prepend(wp_context->route_list, route);
+
+	return TRUE;
+}
+
 static void wispr_portal_request_portal(struct connman_wispr_portal_context *wp_context)
 {
 	DBG("");
 
 	wp_context->request_id = g_web_request_get(wp_context->web,
 					wp_context->status_url,
-					wispr_portal_web_result, wp_context);
+					wispr_portal_web_result,
+					wispr_route_request,
+					wp_context);
 
 	if (wp_context->request_id == 0)
 		wispr_portal_error(wp_context);
+	else
+		wispr_portal_context_ref(wp_context);
 }
 
 static gboolean wispr_input(const guint8 **data, gsize *length,
@@ -432,16 +597,53 @@ static gboolean wispr_input(const guint8 **data, gsize *length,
 	return FALSE;
 }
 
-static void wispr_portal_request_wispr_login(struct connman_service *service,
-				connman_bool_t success,
-				const char *ssid, int ssid_len,
-				const char *username, const char *password,
-				void *user_data)
+static void wispr_portal_browser_reply_cb(struct connman_service *service,
+					connman_bool_t authentication_done,
+					const char *error, void *user_data)
 {
 	struct connman_wispr_portal_context *wp_context = user_data;
 
 	DBG("");
 
+	wp_context = wispr_portal_context_unref(wp_context);
+	if (service == NULL || wp_context == NULL)
+		return;
+
+	if (authentication_done == FALSE) {
+		wispr_portal_error(wp_context);
+		free_wispr_routes(wp_context);
+		return;
+	}
+
+	/* Restarting the test */
+	__connman_wispr_start(service, wp_context->type);
+}
+
+static void wispr_portal_request_wispr_login(struct connman_service *service,
+				connman_bool_t success,
+				const char *ssid, int ssid_len,
+				const char *username, const char *password,
+				gboolean wps, const char *wpspin,
+				const char *error, void *user_data)
+{
+	struct connman_wispr_portal_context *wp_context = user_data;
+
+	DBG("");
+
+	wp_context = wispr_portal_context_unref(wp_context);
+	if (wp_context == NULL)
+		return;
+
+	if (error != NULL && g_strcmp0(error,
+			"net.connman.Agent.Error.LaunchBrowser") == 0) {
+		wispr_portal_context_ref(wp_context);
+
+		__connman_agent_request_browser(service,
+				wispr_portal_browser_reply_cb,
+				wp_context->redirect_url, wp_context);
+		return;
+	}
+
 	g_free(wp_context->wispr_username);
 	wp_context->wispr_username = g_strdup(username);
 
@@ -453,6 +655,8 @@ static void wispr_portal_request_wispr_login(struct connman_service *service,
 					"application/x-www-form-urlencoded",
 					wispr_input, wispr_portal_web_result,
 					wp_context);
+	if (wp_context->request_id != 0)
+		wispr_portal_context_ref(wp_context);
 
 	connman_wispr_message_init(&wp_context->wispr_msg);
 }
@@ -490,10 +694,14 @@ static gboolean wispr_manage_message(GWebResult *result,
 
 		wp_context->wispr_result = CONNMAN_WISPR_RESULT_LOGIN;
 
+		wispr_portal_context_ref(wp_context);
+
 		if (__connman_agent_request_login_input(wp_context->service,
 					wispr_portal_request_wispr_login,
-					wp_context) != -EIO)
+					wp_context) != -EINPROGRESS) {
+			wispr_portal_context_unref(wp_context);
 			wispr_portal_error(wp_context);
+		}
 
 		break;
 	case 120: /* Falling down */
@@ -524,24 +732,6 @@ static gboolean wispr_manage_message(GWebResult *result,
 	return FALSE;
 }
 
-static void wispr_portal_browser_reply_cb(struct connman_service *service,
-					connman_bool_t authentication_done,
-					void *user_data)
-{
-	struct connman_wispr_portal_context *wp_context = user_data;
-
-	if (service == NULL || wp_context == NULL)
-		return;
-
-	if (authentication_done == FALSE) {
-		wispr_portal_error(wp_context);
-		return;
-	}
-
-	/* Restarting the test */
-	__connman_wispr_start(service, wp_context->type);
-}
-
 static gboolean wispr_portal_web_result(GWebResult *result, gpointer user_data)
 {
 	struct connman_wispr_portal_context *wp_context = user_data;
@@ -553,6 +743,10 @@ static gboolean wispr_portal_web_result(GWebResult *result, gpointer user_data)
 
 	DBG("");
 
+	wp_context = wispr_portal_context_unref(wp_context);
+	if (wp_context == NULL)
+		return FALSE;
+
 	if (wp_context->request_id == 0)
 		return FALSE;
 
@@ -560,6 +754,8 @@ static gboolean wispr_portal_web_result(GWebResult *result, gpointer user_data)
 		g_web_result_get_chunk(result, &chunk, &length);
 
 		if (length > 0) {
+			wispr_portal_context_ref(wp_context);
+
 			g_web_parser_feed_data(wp_context->wispr_parser,
 								chunk, length);
 			return TRUE;
@@ -585,15 +781,21 @@ static gboolean wispr_portal_web_result(GWebResult *result, gpointer user_data)
 		if (g_web_result_get_header(result, "X-ConnMan-Status",
 								&str) == TRUE)
 			portal_manage_status(result, wp_context);
-		else
+		else {
+			wispr_portal_context_ref(wp_context);
+
 			__connman_agent_request_browser(wp_context->service,
 					wispr_portal_browser_reply_cb,
 					wp_context->redirect_url, wp_context);
+		}
 
 		break;
 	case 302:
-		if (g_web_result_get_header(result, "Location",
-						&redirect) == FALSE) {
+		if (g_web_supports_tls() == FALSE ||
+				g_web_result_get_header(result, "Location",
+							&redirect) == FALSE) {
+			wispr_portal_context_ref(wp_context);
+
 			__connman_agent_request_browser(wp_context->service,
 					wispr_portal_browser_reply_cb,
 					wp_context->status_url, wp_context);
@@ -605,9 +807,13 @@ static gboolean wispr_portal_web_result(GWebResult *result, gpointer user_data)
 		wp_context->redirect_url = g_strdup(redirect);
 
 		wp_context->request_id = g_web_request_get(wp_context->web,
-				redirect, wispr_portal_web_result, wp_context);
+				redirect, wispr_portal_web_result,
+				wispr_route_request, wp_context);
+		if (wp_context->request_id != 0)
+			wispr_portal_context_ref(wp_context);
 
 		goto done;
+	case 400:
 	case 404:
 		if (__connman_service_online_check_failed(wp_context->service,
 							wp_context->type) == 0)
@@ -618,6 +824,7 @@ static gboolean wispr_portal_web_result(GWebResult *result, gpointer user_data)
 		break;
 	}
 
+	free_wispr_routes(wp_context);
 	wp_context->request_id = 0;
 done:
 	wp_context->wispr_msg.message_type = -1;
@@ -630,14 +837,12 @@ static void proxy_callback(const char *proxy, void *user_data)
 
 	DBG("proxy %s", proxy);
 
+	wp_context = wispr_portal_context_unref(wp_context);
+	if (wp_context == NULL)
+		return;
+
 	wp_context->token = 0;
 
-	if (proxy == NULL)
-		proxy = getenv("http_proxy");
-
-	if (getenv("CONNMAN_WEB_DEBUG"))
-		g_web_set_debug(wp_context->web, web_debug, "WEB");
-
 	if (proxy != NULL && g_strcmp0(proxy, "DIRECT") != 0)
 		g_web_set_proxy(wp_context->web, proxy);
 
@@ -655,8 +860,18 @@ static void proxy_callback(const char *proxy, void *user_data)
 	wispr_portal_request_portal(wp_context);
 }
 
+static gboolean no_proxy_callback(gpointer user_data)
+{
+	struct connman_wispr_portal_context *wp_context = user_data;
+
+	proxy_callback("DIRECT", wp_context);
+
+	return FALSE;
+}
+
 static int wispr_portal_detect(struct connman_wispr_portal_context *wp_context)
 {
+	enum connman_service_proxy_method proxy_method;
 	enum connman_service_type service_type;
 	char *interface = NULL;
 	char **nameservers = NULL;
@@ -692,22 +907,28 @@ static int wispr_portal_detect(struct connman_wispr_portal_context *wp_context)
 
 	if_index = connman_inet_ifindex(interface);
 	if (if_index < 0) {
+		DBG("Could not get ifindex");
 		err = -EINVAL;
 		goto done;
 	}
 
 	nameservers = connman_service_get_nameservers(wp_context->service);
 	if (nameservers == NULL) {
+		DBG("Could not get nameservers");
 		err = -EINVAL;
 		goto done;
 	}
 
 	wp_context->web = g_web_new(if_index);
 	if (wp_context->web == NULL) {
+		DBG("Could not set up GWeb");
 		err = -ENOMEM;
 		goto done;
 	}
 
+	if (getenv("CONNMAN_WEB_DEBUG"))
+		g_web_set_debug(wp_context->web, web_debug, "WEB");
+
 	if (wp_context->type == CONNMAN_IPCONFIG_TYPE_IPV4) {
 		g_web_set_address_family(wp_context->web, AF_INET);
 		wp_context->status_url = STATUS_URL_IPV4;
@@ -719,13 +940,23 @@ static int wispr_portal_detect(struct connman_wispr_portal_context *wp_context)
 	for (i = 0; nameservers[i] != NULL; i++)
 		g_web_add_nameserver(wp_context->web, nameservers[i]);
 
-	wp_context->token = connman_proxy_lookup(interface,
-					wp_context->status_url,
-					wp_context->service,
-					proxy_callback, wp_context);
-	if (wp_context->token == 0)
-		err = -EINVAL;
+	proxy_method = connman_service_get_proxy_method(wp_context->service);
 
+	if (proxy_method != CONNMAN_SERVICE_PROXY_METHOD_DIRECT) {
+		wp_context->token = connman_proxy_lookup(interface,
+						wp_context->status_url,
+						wp_context->service,
+						proxy_callback, wp_context);
+
+		if (wp_context->token == 0) {
+			err = -EINVAL;
+			goto done;
+		}
+	} else {
+		g_timeout_add_seconds(0, no_proxy_callback, wp_context);
+	}
+
+	wispr_portal_context_ref(wp_context);
 done:
 	g_strfreev(nameservers);
 
@@ -769,9 +1000,9 @@ int __connman_wispr_start(struct connman_service *service,
 
 	/* If there is already an existing context, we wipe it */
 	if (wp_context != NULL)
-		free_connman_wispr_portal_context(wp_context);
+		wispr_portal_context_unref(wp_context);
 
-	wp_context = g_try_new0(struct connman_wispr_portal_context, 1);
+	wp_context = create_wispr_portal_context();
 	if (wp_context == NULL)
 		return -ENOMEM;
 
